%
%    Copyright (C) 2008  Colin Caprani - www.colincaprani.com
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.

function OutMat = CBA(L,E,I,R,LM,clr)
% This is the main Continuous Beam Analysis function
% It uses the stiffness method to solve for member end moments, shears,
% rotations and deflections, and then uses member formulae to find values
% along the members. It returns a matrix of size [NoSpansx100 4] where the
% four columns are distance, moment, shear and deflection. Values are 
% calculated at 100 points along each beam member. Plots of moment, shear
% and deflection are also output.

N = length(L); % number of spans
E = E*1e6; % kN/mm^2 to kN/m^2
I = I./1e12; % mm4 to m4

n_nodes = N+1;
ndof = 2*n_nodes;
k_sys = zeros(ndof,ndof); % initialize the matrix to be zero
m = zeros(ndof,1);
f_mbr = zeros(4*N,1);

for i = 1:N
    j = i+1;
    kb = k_beam(L(i),I(i),E);
    dof_i = 2*i-1;
    % assign to global stiffness matrix
    for k = 1:4
        for l = 1:4
            k_sys(dof_i+k-1,dof_i+l-1) = k_sys(dof_i+k-1,dof_i+l-1) + kb(k,l);
        end
    end
    RLM = LM_span(LM,i);
    m( dof_i:(dof_i+3) ) = m( dof_i:(dof_i+3) ) - CNL(RLM,L(i));
end

% apply constraints
for i = 1:ndof
    if R(i) < 0 
        for j = 1:ndof
           k_sys(i,j) = 0; k_sys(j,i) = 0;
        end
        k_sys(i,i) = 1;
        m(i) = 0; % here for specified displacements
    end
    if R(i) > 0
        k_sys(i,i) = k_sys(i,i)+ R(i); % spring supports
    end
end
%solve
d = k_sys\m;
%calculate member end actions
for i = 1:N
    kb = k_beam(L(i),I(i),E);
    dof_i = 2*i-1;
    d_mbr = d(dof_i:(dof_i+3));
    dof_i = 4*i-3;
    
    RLM = LM_span(LM,i); % Reduced Load Matrix for this span
    fmbr = kb*d_mbr + CNL(RLM,L(i));
    f_mbr(dof_i:(dof_i+3)) = fmbr;
    [xt(:,i) Mt(:,i) Vt(:,i) dt(:,i)] = mbr_values(fmbr,d_mbr,RLM,L(i),E,I(i));
    xt(:,i) = xt(:,i) + sum(L(1:i-1));
end

[np N] = size(xt);
for i = 1:N
    first = (i-1)*np + 1;
    last = i*np;
    OutMat(first:last,1) = xt(:,i);
    OutMat(first:last,2) = Mt(:,i);
    OutMat(first:last,3) = Vt(:,i);
    OutMat(first:last,4) = dt(:,i);
end

Lght = sum(L);
subplot(3,1,1); hold on; grid on;
plot(xt,-Mt,clr); plot([0 Lght],[0 0],'k','linewidth',2);
ylabel('Bending Moment (kNm)');

subplot(3,1,2); hold on; grid on;
plot(xt,Vt,clr); plot([0 Lght],[0 0],'k','linewidth',2);
ylabel('Shear Force (kN)');

subplot(3,1,3); hold on; grid on;
plot(xt,dt.*1e3,clr); plot([0 Lght],[0 0],'k','linewidth',2);
ylabel('Deflection (mm)');
xlabel('Distance along beam (m)'); 