%
%    Copyright (C) 2008  Colin Caprani - www.colincaprani.com
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.

% ---------------------------------------
% Continuous Beam Analysis - C.C. Caprani
% ---------------------------------------
% This uses the stiffness method to determine the rotations at each 
% support. These are then used to determine the member end forces. Exact 
% expressions are then used to determine the distribution of shear, moment, 
% rotation and deflection along each member.
%
% The input data is to be in the following format:
% L - a vector of the span lengths (m)
% E - Modulous of Elasticy for the problem - a scalar value (kN/mm2)
% I - a vector of second moment of area; one for each span (mm4)
% R - a vector of restraints for each node:
%     - Each node has 2 degress of freedom, vertical deflection and 
%       rotation, in that order, for each node;
%     - Restraint is indicated by a "-1" value, for the dof of the node;
%     - Unrestrained degree of freedoms are indicated by a "0";
%     - Supports with a stiffness (kN/m or kN/rad) are indicated by a
%       positive value of the stiffness, k: i.e. "+k"
% LM - a matrix representing the loads. Each row represents a different 
%      load and must be in the following format:
%      Span No. | Load Type | Load Value | Distance a | Load Cover c
%      Load Types are: 1 - Uniformly Distributed Loads, which only have 
%                          a load value; set distance a to "0"
%                      2 - Point Loads, located at "a" from the left end of
%                          the span.
%                      3 - Partial UDLs, starting at "a" for a distance of
%                          "c" where L >= a+c;
%                      4 - Moment Load, located at "a".
%      Load units are to be in kN, kN/m and metres.
% C - a Matlab colour to give to the plots: 'k', 'g' etc.

% Example 1: two-span beam with a UDL of 20 kN/m on each span
L = [7.5,7];
E = 30;
I = [600e7,600e7];
R = [-1,0,-1,0,-1,0];
LM = [1,1,20,0,0;
      2,1,20,0,0];
OutMat = CBA(L,E,I,R,LM,'r');

% Example 2: two-span beam with a PL of 50 kN at a = 3 on the first span, a
% moment load of 50 kNm at a = 2m on the second and a fixed support at the 
% right end
L = [5,5];
E = 30;
I = [600e7,600e7];
R = [-1,0,-1,0,-1,-1];
LM = [1,2,50,3,0;
      2,4,50,2,0];
OutMat = CBA(L,E,I,R,LM,'r');

% Example 2a: Check on this program against Wilson et al VDB.
L = [5,7];
E = 30;
I = [400e7,600e7];
R = [-1,0,-1,0,-1,-1];
LM = [1,2,50,3,0;
      2,1,20,0,0];
OutMat = CBA(L,E,I,R,LM,'r');

% Example 3: Timber floor joists at 300 mm c/c
L = [4.5];
E = 8;
I = [2.93e7];
R = [-1,0,-1,0];
s = 0.3; 
Pg = 5.65; Pq = 1.6;
wg = 0.7; wq = 1.5;
Pg = Pg*s; Pq = Pq*s;
wg = wg*s; wq = wq*s;
LMg = [1,1,wg,0,0;
       1,2,Pg,0.3,0];
LMq = [1,1,wq,0,0;
       1,2,Pq,0.3,0];
LMs = addLM(LMg,LMq);
OutMat = CBA(L,E,I,R,LMs,'r');
OutMat = CBA(L,E,I,R,LMg,'r');

% Example 4: 3-span beam with differential settlement under self weight
L = [15,15,15];
E = 30;
I = [500e8,500e8,500e8];
R = [-1,0,1e8,0,1e8,0,-1,0];
LM = [1,1,20,0,0;
      2,1,20,0,0;
      3,1,20,0,0];
OutMat = CBA(L,E,I,R,LM,'r');

% Example 5: 3-span beam in a subframe. Columns are 4 m high, and 300 mm
% square. Hence the roational stiffness at each joint is 2 x (4EI)/L:
% 8*30*(300^4)/4000 *10e3 = 486e9
L = [6,8,6];
E = 30;
I = [50e8,50e8,50e8];
R = [-1,486e9,-1,486e9,-1,486e9,-1,486e9];
LM = [1,1,10,0,0;
      2,1,20,0,0;
      3,1,10,0,0];
OutMat = CBA(L,E,I,R,LM,'r');

% Example 6: 4-span beam. Determining the equivalent loads applied by a
% post-tensioned cable
L = [6,8,6,8];
E = 30;
I = [100e8,100e8,100e8,100e8];
R = [-1,0,-1,0,-1,0,-1,0,-1,0];
LM = [1,3,20,0,1;
      1,3,-10,1,4;
      1,3,20,5,1;
      1,4,50,0,0;
      2,3,20,0,1.5;
      2,3,-10,1.5,5;
      2,3,20,6.5,1.5;
      3,3,20,0,1;
      3,3,-10,1,4;
      3,3,20,5,1;
      4,3,20,0,1.5;
      4,3,-10,1.5,5;
      4,3,20,6.5,1.5;
      4,4,-75,8,0;];
OutMat = CBA(L,E,I,R,LM,'r');

% Example 7 - Superposition for load balancing. Using the addLM subroutine,
% two load matrices may be superimposed. From Example 5, the service load
% is taken and added, then the prestress equivalent loads are added. Both
% load cases are analysed seperately and the combined LM analysed: each set
% of results are displayed seperately.
L = [6,8];
E = 30;
I = [100e8,100e8];
R = [-1,0,-1,0,-1,0];
LMg = [1,1,20,0,0;
       2,1,20,0,0];
LMq = [1,1,30,0,0;
       2,1,30,0,0];
LMp = [1,4,100,0,0;
       2,4,-100,8,0];

OutMat = CBA(L,E,I,R,LMp,'r');
LMt = addLM(LMg,LMp); % transfer moments
figure; 
OutMat_t = CBA(L,E,I,R,LMt,'r');
LMs = addLM(LMt,LMq); % service moments
figure; 
OutMat_s = CBA(L,E,I,R,LMs,'r');

% Example 8 - Superposition for load balancing. Using the addLM subroutine,
% two load matrices may be superimposed. From Example 5, the service load
% is taken and added, then the prestress equivalent loads are added. Both
% load cases are analysed seperately and the combined LM analysed: each set
% of results are displayed seperately.
L = [6,8,6,8];
E = 30;
I = [100e8,100e8,100e8,100e8];
R = [-1,0,-1,0,-1,0,-1,0,-1,0];
LMg = [1,1,20,0,0;
       2,1,20,0,0;
       3,1,20,0,0;
       4,1,20,0,0];
LMq = [1,1,30,0,0;
       2,1,30,0,0;
       3,1,20,0,0;
       4,1,30,0,0];
LMp = [1,3,20,0,1;
      1,3,-25,1,4;
      1,3,20,5,1;
      1,4,10,0,0;
      2,3,20,0,1.5;
      2,3,-25,1.5,5;
      2,3,20,6.5,1.5;
      3,3,20,0,1;
      3,3,-25,1,4;
      3,3,20,5,1;
      4,3,20,0,1.5;
      4,3,-25,1.5,5;
      4,3,20,6.5,1.5;
      4,4,-15,8,0;]; 
LMt = addLM(LMg,LMp); % transfer moments
OutMat_t = CBA(L,E,I,R,LMt,'r');
%figure; 
%OutMat = CBA(L,E,I,R,LMp,'r');
LMs = addLM(LMg,LMq);
LMs = addLM(LMs,LMp); % service moments
figure; 
OutMat_s = CBA(L,E,I,R,LMs,'r');

